@@ -1,29 +1,26 @@
 package Kwiki::Archive;
-use strict;
-use warnings;
-use Kwiki::Plugin '-Base';
+use Kwiki::Plugin -Base;
 
 const class_id => 'archive';
 const class_title => 'Page Archive';
-
-field 'user_name';
+const show_revisions => 1;
 
 sub register {
     my $registry = shift;
-    $registry->add(page_hook_store => 'commit');
+    $registry->add(hook => 'page:store', post => 'commit');
 }
 
 sub init {
     $self->use_class('pages');
     if ($self->empty) {
         $self->generate;
-        $self->user_name('kwiki-install');
         $self->commit_all;
     };
 }
 
 sub empty {
-    io($self->plugin_directory)->empty;
+    my $dir = io($self->plugin_directory);
+    $dir->exists and $dir->empty;
 }
 
 sub generate {
@@ -40,19 +37,21 @@ sub commit_all {
 
 sub page_properties {
     my $page = shift;
-    return {
-        edit_by => $self->user_name || $page->metadata->edit_by,
-        edit_time => $page->metadata->edit_time || scalar(gmtime),
-        edit_unixtime => $page->metadata->edit_unixtime || scalar(time)
-    };
+    my $properties = $page->metadata->to_hash;
+    $properties->{edit_by} ||= '';
+    $properties->{edit_time} ||= scalar(gmtime);
+    $properties->{edit_unixtime} ||= scalar(time);
+    return $properties;
 }
 
 sub revision_number {
     $self->history(shift)->[0]->{revision_id} || 0;    
 }
 
-
-1;
+sub revision_numbers {
+    my $page = shift;
+    [map $_->{revision_id}, @{$self->history($page)}];
+}
 
 __DATA__
 
